from django.db import models
from django.utils.translation import gettext_lazy as _

from djmoney.models.fields import MoneyField
from localflavor.generic.models import IBANField, BICField

from aleksis.core.mixins import ExtensibleModel
from aleksis.core.models import Group, Person


class Organisation(ExtensibleModel):
    """An organisation (NPO/NGO) managed in the app."""

    name = models.CharField(verbose_name=_("Organisation mane"), max_length=64)

    class Meta:
        constraints = [
            models.UniqueConstraint(fields=["site_id", "name"], name="unique_org_name_per_site"),
        ]


class LegalEntity(ExtensibleModel):
    """A legal entity that can exercise membership rights or other relations."""

    name = models.CharField(verbose_name=_("Entity name"), max_length=64)
    related_group = models.OneToOneField(Group, verbose_name=_("Related group"), related_name="related_legal_entity", null=True, on_delete=models.SET_NULL)

    class Meta:
        constraints = [
            models.UniqueConstraint(fields=["site_id", "name"], name="unique_entity_name_per_site"),
        ]


class MembershipType(ExtensibleModel):
    """A defined type of membership with a name and rules."""

    caption = models.CharField(verbose_name=_("Caption"), max_length=64)
    description = models.CharField(verbose_name=_("Description"), max_length=255, blank=True, null=True)

    organisations = models.ManyToManyField(Organisation, verbose_name=_("Applicable for organisations"), related_name="membership_types")

    class Meta:
        constraints = [
            models.UniqueConstraint(fields=["site_id", "caption"], name="unique_type_name_per_site"),
        ]


class SEPAMandate(ExtensibleModel):
    """A SEPA direct debit mandate for membership fees."""

    account_holder = models.CharField(verbose_name=_("Account holder"), max_length=64)

    iban = IBANField(verbose_name=_("IBAN of bank account"))
    bic = BICField(verbose_name=_("BIC/SWIFT code of bank"))

    date = models.DateField(verbose_name=_("Date mandate was granted"))


class Membership(ExtensibleModel):
    """A membership of a person or legal entity."""

    RECURRENCE_CHOICES = (
        ("daily", _("daily")),
        ("weekly", _("weekly")),
        ("monthly", _("monthly")),
        ("quarterly", _("quarterly")),
        ("yearly", _("yearly")),
    )

    person = models.ForeignKey(Person, verbose_name=_("Responsible person"), related_name="org_memberships", on_delete=models.CASCADE)
    legal_entity = models.ForeignKey(LegalEntity, verbose_name=_("Legal entity represented by the person"), related_name="org_memberships", null=True, on_delete=models.CASCADE)
    organisation = models.ForeignKey(Organisation, verbose_name=_("Membership organisation"), related_name="memberships", on_delete=models.CASCADE)

    code = models.CharField(verbose_name=_("Membership code/number"), blank=True, max_length=255)

    begin = models.DateField(verbose_name=_("Beginning date of membership"))
    end = models.DateField(verbose_name=_("End date of membership"), null=True)

    membership_type = models.ForeignKey(MembershipType, verbose_name=_("Membership type"), related_name="memberships", null=True, on_delete=models.SET_NULL)
    fees = MoneyField(verbose_name=_("Membership fees"), default=0, max_digits=8, decimal_places=2)
    fees_recurrence = models.CharField(verbose_name=("Fee recurrence"), choices=RECURRENCE_CHOICES, default="yearly", max_length=9)

    sepa_mandate = models.OneToOneField(SEPAMandate, verbose_name=_("Linked SEPA mandate"), null=True, on_delete=models.SET_NULL)
