from django.urls import reverse_lazy
from django.utils.decorators import method_decorator
from django.utils.translation import gettext_lazy as _
from django.views.decorators.cache import never_cache
from django.views.generic.detail import DetailView

from django_tables2 import SingleTableView
from rules.contrib.views import PermissionRequiredMixin

from aleksis.core.mixins import AdvancedCreateView, AdvancedEditView

from .forms import OrganisationForm, MembershipTypeForm
from .models import Organisation, MembershipType
from .tables import OrganisationTable, MembershipTypeTable

class OrganisationsListView(PermissionRequiredMixin, SingleTableView):
    """Table of all organisations."""

    model = Organisation
    table_class = OrganisationTable
    permission_required = "vereinsmeier.view_organisation_rule"
    template_name = "vereinsmeier/organisation/list.html"


class OrganisationDetailView(PermissionRequiredMixin, DetailView):
    """Detail view of one organisation."""

    context_object_name = "organisation"
    template_name = "vereinsmeier/organisation/full.html"
    permission_required = "vereinsmeier.view_organisation_rule"

    def get_queryset(self):
        return Organisation.objects.all()


@method_decorator(never_cache, name="dispatch")
class OrganisationEditView(PermissionRequiredMixin, AdvancedEditView):
    """Edit view for a single organisation."""

    model = Organisation
    form_class = OrganisationForm
    template_name = "vereinsmeier/organisation/edit.html"
    template_name = "vereinsmeier.edit_organisation_rule"
    success_url = reverse_lazy("organisations")
    success_message = _("The organisation was successfully saved.")


@method_decorator(never_cache, name="dispatch")
class OrganisationCreateView(PermissionRequiredMixin, AdvancedCreateView):
    """Create view for a single organisation."""

    model = Organisation
    form_class = OrganisationForm
    template_name = "vereinsmeier/organisation/create.html"
    permission_required = "vereinsmeier.create_organisation_rule"
    success_url = reverse_lazy("organisations")
    success_message = _("The organisation was successfully created.")

class MembershipTypesListView(PermissionRequiredMixin, SingleTableView):
    """Table of all membership types."""

    model = MembershipType
    table_class = MembershipTypeTable
    permission_required = "vereinsmeier.view_membership_type_rule"
    template_name = "vereinsmeier/membership_type/list.html"


class MembershipTypeDetailView(PermissionRequiredMixin, DetailView):
    """Detail view of one membership type."""

    context_object_name = "membership_type"
    template_name = "vereinsmeier/membership_type/full.html"
    permission_required = "vereinsmeier.view_membership_type_rule"

    def get_queryset(self):
        return MembershipType.objects.all()


@method_decorator(never_cache, name="dispatch")
class MembershipTypeEditView(PermissionRequiredMixin, AdvancedEditView):
    """Edit view for a single membership type."""

    model = MembershipType
    form_class = MembershipTypeForm
    template_name = "vereinsmeier/membership_type/edit.html"
    permission_required = "vereinsmeier.edit_membership_type_rule"
    success_url = reverse_lazy("membership_types")
    success_message = _("The membership type was successfully saved.")


@method_decorator(never_cache, name="dispatch")
class MembershipTypeCreateView(PermissionRequiredMixin, AdvancedCreateView):
    """Create view for a single membership type."""

    model = MembershipType
    form_class = MembershipTypeForm
    template_name = "vereinsmeier/membership_type/create.html"
    permission_required = "vereinsmeier.create_membership_type_rule"
    success_url = reverse_lazy("membership_types")
    success_message = _("The membership type  was successfully created.")
